/* tu58.h - TU58 and RSP definitions					*/
/*									*/
/* Copyright (C) 1994 by Robert Armstrong				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify	*/
/* it under the terms of the GNU General Public License as published by	*/
/* the Free Software Foundation; either version 2 of the License, or	*/
/* (at your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful, but	*/
/* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-	*/
/* ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General	*/
/* Public License for more details.					*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, visit the website of the Free	*/
/* Software Foundation, Inc., www.gnu.org.				*/



/* TU58 General Parameters */
#define MAXBLOCK	511	/* maximum block number (normal mode!)	*/
#define BLOCKSIZE	512	/* size of a virtual tape block		*/
#define DATASIZE	128	/* size of a RSP data packet		*/

/* TU58 Timeouts */
/*   All timeout values are in ticks of the BIOS clock (about 55mS).	*/
/* The long timeout is used for operations which must move the tape,	*/
/* and the short timeout is for everything else...			*/
#define SHORTWAIT	  18	/* approximately 1 second		*/
#define LONGWAIT	1091	/*    "    "     60 seconds		*/

/* TU58 Packet Types */
/*   NOTE: in the TU58 documentation, these are referred to as "flags"!	*/
#define F_DATA		0001	/* data packets for read/write		*/
#define F_CONTROL	0002	/* control (command) packet		*/
#define F_INITIALIZE	0004	/* forces controller initialization	*/
#define F_CONTINUE	0020	/* continue with the next data packet	*/

/* TU58 Command OpCodes */
#define O_NOP		0000	/* no operation (returns an END packet)	*/
#define O_READ		0002	/* reads data from the tape		*/
#define O_WRITE		0003	/* writes data to the tape		*/
#define O_POSITION	0005	/* positions the tape at a given block	*/
#define O_DIAGNOSE	0007	/* executes built in self test		*/
#define O_END		0100	/* acknowledgement (end) packet type	*/

/* TU58 Command Modifiers */
#define M_SAM		0200	/* selects special addressing mode	*/
#define M_CHECK		0001	/* decreased sensitivity (read command)	*/
				/* read and verify (write command)	*/

/* TU58 Error Codes */
#define	E_SUCCESS	0000	/* operation completed without errors	*/
#define E_WRETRY	0001	/* success, but retries were needed	*/
#define E_POSTFAIL	0377	/* self test failed			*/
#define E_EOT		0376	/* partial operation - end of tape	*/
#define E_BADUNIT	0370	/* unit number not 0 or 1		*/
#define E_NOTAPE	0367	/* no cartridge in drive		*/
#define E_WLOCK		0365	/* tape is write protected		*/
#define E_DATACHECK	0357	/* bad data (checksum failure)		*/
#define E_SEEKFAIL	0340	/* block not found (bad tape format?)	*/
#define E_JAMMED	0337	/* motor stopped (jammed cartridge?)	*/
#define E_BADOPCODE	0320	/* bad opcode in command packet		*/
#define E_BADBLOCK	0311	/* block number .GT. 511		*/
/*   These error codes are never returned by the TU58, but are used for	*/
/* internal conditions.  They must not overlap the real TU58 codes!	*/
#define E_TIMEOUT	0100	/* no response from the TU58		*/
#define E_BADTYPE	0101	/* TU58 responded with the wrong packet	*/
#define E_BADLENGTH	0102	/* TU58 data length was not expected	*/
#define E_CHECKSUM	0177	/* packet checksum doesn't verify	*/

/* TU58 Command Packet Structure */
/*   NOTE: this does not include the type (flag) byte, the message byte	*/
/* count byte, nor the two checksum bytes!				*/
struct _CMDPKT {
  BYTE opcode;			/* opcode for this command (O_xxx)	*/
  BYTE modifier;		/* modifier bits for opcode (M3_xxx)	*/
  BYTE unit;			/* must selects unit 0 or 1		*/
  BYTE switches;		/* additional modifiers (M5_xxx)	*/
  BYTE seq_l, seq_h;		/* packet sequence number (not used!)	*/
  BYTE dbc_l, dbc_h;		/* data byte count for read/write	*/
  BYTE lbn_l, lbn_h;		/* logical block number			*/
};
#define CMDPKTLEN	sizeof(struct _CMDPKT)
typedef struct _CMDPKT CMDPKT;

/* TU58 Acknowledgement (End) Packet Structure */
/*   NOTE: this also does not include the type (flag) byte, the message	*/
/* byte count byte, nor the two checksum bytes.  In the TU58 literature	*/
/* this packet is referred to as a "END" packet...			*/
struct _ENDPKT {
  BYTE opcode;			/* always O_END				*/
  BYTE success;			/* success or error code (E_xxx)	*/
  BYTE unit;			/* unit number from original command	*/
  BYTE switches;		/* switches (unused in end packets)	*/
  BYTE seq_l, seq_h;		/* packet sequence number (unused)	*/
  BYTE abc_l, abc_h;		/* actual byte count for read/write	*/
  BYTE sts_l, sts_h;		/* status flag bits (almost unused!)	*/
};
#define ENDPKTLEN	sizeof(struct _ENDPKT)
typedef struct _ENDPKT ENDPKT;


/* TUInitialize - establish two way communications with the drive */
BOOLEAN TUInitialize (void);
/* TUDiagnose - ask the drive to execute its built in self test	*/
int TUDiagnose (void);
/* TUSend - send raw packets to the drive */
void TUSend (BYTE, BYTE far *, BYTE);
/* TUReceive - receive raw packets from the drive */
int TUReceive (BYTE, BYTE far *, BYTE, int);
/* TUEnd - wait for and receive an acknowledgement packet */
int TUEnd (int);
/* TUPosition - position the tape to a specific block (aka seek) */
int TUPosition (int, int);
/* TURead - read one 512 byte logical block from the tape */
int TURead (int, int, BYTE far *);
/* TUWrite - write one 512 byte logical block to the tape */
int TUWrite (int, int, BYTE far *, BOOLEAN);
